package com.fujitsu.fnc.nms.fenics.gui.gcf;

import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.SwingUtilities;

/**
  A dialog that displays a progress bar for supporting progress
  reporting of Netsmart Command Request and Progress Status action
  commands.

  <p>Instances have a "Cancel" button which lets the user cancel any
  pending action commands, which always execute in a background
  thread.  

  @see FAbstractAction
  @see FWorker

  */
public class FProgressBarDialog extends JFrame
{
  /**
    No-arg constructor.

    */
  public FProgressBarDialog()
  {
    this("Action command execution in progress");
  }


  /**
    Constructor.

    @param msg the message text that displays in the progress dialog.

    */
  public FProgressBarDialog(String msg)
  {
    this("Action Command in Progress", msg);
  }


  /**
    Constructor.

    @param title the title of this progress dialog, which displays 
    in the frame's title area.

    @param msg the message text that displays in the progress dialog.

    */
  public FProgressBarDialog(String title, String msg)
  {
    super(title);

    // When we move to a more contemporary release of Swing, such as
    // JDK 2 release, we might want to use another one of the
    // JProgressBar constructors.
    //
    progressBar = new JProgressBar();
    progressBar.setPreferredSize(new Dimension(250, 17));

    // Uncomment the following line when we move to Swing 1.1; that
    // release's JProgressBar implementation supports the string
    // painted.
//     progressBar.setStringPainted(true);

    progressLabel = new JLabel(msg);

    GridBagLayout layout = new GridBagLayout();
    GridBagConstraints constraints = new GridBagConstraints();

    Container contentPane = getContentPane();
    contentPane.setLayout(layout);

    constraints.gridwidth = GridBagConstraints.REMAINDER;
    constraints.insets = new Insets(15, 15, 10, 15);
    layout.setConstraints(progressLabel, constraints);
    contentPane.add(progressLabel);

    constraints.gridy = 1;
    constraints.gridwidth = GridBagConstraints.REMAINDER;
    layout.setConstraints(progressBar, constraints);
    contentPane.add(progressBar);

    // Had trouble getting these buttons to line up with the
    // GridBagLayout.  So I added the buttons to a panel and 
    // added the panel to the GridBagLayout.
    //
//     constraints.gridy = 2;
//     constraints.gridwidth = 1;
//     layout.setConstraints(dismissButton, constraints);
//     contentPane.add(dismissButton);

//     constraints.gridx = GridBagConstraints.RELATIVE;
//     constraints.gridwidth = GridBagConstraints.REMAINDER;
//     layout.setConstraints(cancelButton, constraints);
//     contentPane.add(cancelButton);

    JPanel buttonPanel = new JPanel();
    buttonPanel.add(dismissButton);
    buttonPanel.add(cancelButton);

    constraints.gridy = 2;
    layout.setConstraints(buttonPanel, constraints);
    contentPane.add(buttonPanel);

    addDismissListener();
  }


  /**
    Adds an action listener to the "Dismiss" button to close the
    dialog at the user's request.

    */
  private void addDismissListener()
  {
    ActionListener aL = new ActionListener()
      {
	public void actionPerformed(ActionEvent e)
	  {
	    FProgressBarDialog.this.dispose();
	  }
      };
    dismissButton.addActionListener(aL);
  }


  /**
    Gets the preferred size of this progress dialog.

    @return this dialog's preferred size.

    */
  public Dimension getPreferredSize()
  {
    return getMinimumSize();
  }


  /**
    Gets the minimum size of this progress dialog.

    @return the progress dialog's minimum size.

    */
  public Dimension getMinimumSize()
  {
    FontMetrics fm = getFontMetrics(progressLabel.getFont());
    int stringWidth = fm.stringWidth(progressLabel.getText());

    return new Dimension(stringWidth + 2*20, 190);
  }


  /**
    Gets the progress bar used by this progress dialog.  This method
    is useful for getting information about the state of the progress
    bar's model.

    <p>For example, to get the progress bar's model, do 

    <pre>
    FProgressBarDialog d = new FProgressBarDialog();
    d.getProgressBar().getModel();
    </pre>

    @return this dialog's progress bar.

    */
  public JProgressBar getProgressBar()
  {
    return progressBar;
  }


  /**
    Gets the current value stored in this dialog's progress bar.

    @return the current value of the progress bar.

    */
  public int getValue()
  {
    return progressBar.getValue();
  }


  /**
    Sets the value of the progress bar maintained by this progress
    dialog.  

    @param n the value for the progress bar to display.

    */
  public void setValue(final int n)
  {
    Runnable progressUpdater = new Runnable()
      {
	public void run()
	  {
	    progressBar.setValue(n);
	    invalidate();
	    repaint();
	  }
      };
    SwingUtilities.invokeLater(progressUpdater);
  }


  /**
    Sets the text of the message displayed by this progress dialog.

    @param msg the message text.

    */
  public void setString(String msg)
  {
    progressLabel = new JLabel(msg);
    invalidate();
    repaint();
  }


  /**
    Gets the message text displayed in the body of this progress
    dialog.

    @return this progress dialog's message text.  

    */
  public String getString()
  {
    return progressLabel.getText();
  }


  /**
    Adds an action listener to this progress dialog's "Cancel" button.
    Any action listener registered thus should be prepared to cancel
    the <code>FWorker</code> instance about whose execution progress
    this dialog represents.

    @param l the action listener to register.

    */
  public void addActionListener(ActionListener l)
  {
    if (l != null)
      cancelButton.addActionListener(l);
  }


  /**
    Removes a previously registered action listener from this dialog's
    "Cancel" button.

    @param l the action listener to remove.

    */
  public void removeActionListener(ActionListener l)
  {
    if (l != null)
      cancelButton.removeActionListener(l);
  }


  /**
    The message text displayed in the dialog.

    */
  protected JLabel progressLabel;


  /**
    The "Dismiss" button displayed on the dialog.

    */
  JButton dismissButton = new JButton("Dismiss");

  /**
    The "Cancel" button displayed in the dialog.

    */
  protected JButton cancelButton = new JButton("Cancel");


  /**
    The progress bar used in this dialog.

    */
  protected JProgressBar progressBar;

  /**
    The file version information string.

    */
  public static final String sccsid = "@(#) /vob/IEM_Devp_gui/gui/gcf/FProgressBarDialog.java /main/r1.1/r1.1_nmsca04573/6 1999-03-30 12:29:44";


}
